@@ -3,7 +3,6 @@ package App::Netdisco::Daemon::Worker::Manager;
 use Dancer qw/:moose :syntax :script/;
 use Dancer::Plugin::DBIC 'schema';
 
-use App::Netdisco::Util::Device 'is_discoverable';
 use Net::Domain 'hostfqdn';
 use Try::Tiny;
 
@@ -14,7 +13,7 @@ my $fqdn = hostfqdn || 'localhost';
 
 my $role_map = {
   (map {$_ => 'Poller'}
-      qw/discoverall discover arpwalk arpnip macwalk macsuck/),
+      qw/discoverall discover arpwalk arpnip nodenames macwalk macsuck/),
   (map {$_ => 'Interactive'}
       qw/location contact portcontrol portname vlan power/)
 };
@@ -57,10 +56,6 @@ sub worker_body {
       while (my $job = $rs->next) {
           my $jid = $job->job;
 
-          # filter for discover_*
-          next unless is_discoverable($job->device);
-          debug sprintf "mgr (%s): job %s is discoverable", $wid, $jid;
-
           # check for available local capacity
           next unless $self->do('capacity_for', $job->action);
           debug sprintf "mgr (%s): processing node has capacity for job %s (%s)",
